%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This code allows to obtain IDF Curves (Intensity-Duration-Frequency) 
% Curves of annual maximum rainfall events by using the Gumbel PDF. 
% Additionally, the Bernard model is fitted to obtain a mathematical
% expression for the IDF curves.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Inputs
%             
%       Annual Maximum Values of Rainfall Depth (mm) for different 
%       Durations (min or hours)   
%
%        Year     5     10     30   Duration of Rainfall (min or hours)
%        1956    4.5    6.7    7.8  Rainfall Depth (mm).
%        1957    2.6    5.6    8.9  
%          .      .      .      .
%          .      .      .      .
%        2014    3.3    7.8    8.8              
%                             
%       The Rainfall Depth Data is read from .txt File. Rows are years  
%       and columns are different Durations. Rainfall values are 
%       annual maximums associated to an specific duration.

%% Outputs
%               
%     1) A plot for the IDF Curves
%     2) IDF Model proposed by Bernard (1932): [I = (k*T^a)/D^b]
%
%        Where,
%
%           I               is the Rainfall Intensity (mm/h or mm/min).
%           k, a, and b     are the regression coefficients.
%           D               is the Duration (hours or minutes).
%           T               is the Return Period (years).
%
%% References
%       
% Bernard, M. 1932. Formulas for rainfall intensities of long durations. 
% TRANS. ASCE. Vol. 96. pp 592-624. 
% Chow, V., Maidment, D., and Mays, L. 1994. Hidrologa aplicada. 
% Santa Fe de Bogot, Colombia: Mc  Graw-Hill. 584 p.
%              
%% Author's Statement
%
%       Rodrigo Valdes Pineda and Juan B. Valdes. University of Arizona. 
%       Tucson, AZ. 
%       This routine is provided "as is" without warranty of any kind. 
%       Please, respect author's rights, do not attribute it.
%       If you detect any mistake or bug, please contact us to:
%       rvaldes@email.arizona.edu -- jvaldes@email.arizona.edu
%
%% Clear Variables

        clear all
        close all

%% Load Data

% Rainfall Depth Data is read from .txt File. Rows are years  
% and columns are different Durations. Rainfall values are 
% annual maximums associated to an specific duration.

Rainfall_Depth  = importdata('Rainfall_Data.txt');   

% Defining Durations (D) [hours or minutes]; and Return Periods (T) [years]

Duration      = [1 2 6 12 24];                   % Duration [hours]
Return_Period = [5 10 20 30 40 50 60 75 100]';   % Return Period [years]

% Converting Rainfall Depth (mm) to Rainfall Intensity (mm/h)

n                   = length(Rainfall_Depth(:,1)); % Column Dimension
m                   = length(Rainfall_Depth(1,:)); % Row Dimension
t                   = length(Return_Period(:,1));  % Return Period Dimension
Rainfall_Intensity  = zeros(n,m);                  % Storage Matrix

% Rainfall Intensity for Year "i" and Duration "j".

for i = 1 : n;
    for j = 1 : m;
          Rainfall_Intensity(i,j) = Rainfall_Depth(i,j)/Duration(j); 
    end
end 

%% Fitting Gumbel

%  Obtaining Fiting Parameters (Mean, Standard Deviation (S),
%  Shape Parameter (Alpha), and Scale Parameter (u).

% Storage Matrixes

Mean_Rainfall   = zeros(1,m); 
Std_Rainfall    = zeros(1,m);
Alpha_Rainfall  = zeros(1,m);
u_Rainfall      = zeros(1,m);

for i = 1 : n;
    for j = 1 : m;
          Mean_Rainfall(1,j)   = mean(Rainfall_Intensity(:,j));
          Std_Rainfall(1,j)    = std(Rainfall_Intensity(:,j));
          Alpha_Rainfall(1,j)  = 1.282/Std_Rainfall(1,j);
          u_Rainfall(1,j)      = Mean_Rainfall(1,j)-0.450047*Std_Rainfall(1,j);
    end
end
%% Obtaining IDF Tables

%  Storage Matrixes

yT        = zeros(t,1);
IDF_Table = zeros(t,m);

for i = 1 : t;
    for j = 1 : m;
         yT(i,1)        = (-log(log(Return_Period(i,1)/(Return_Period(i,1)-1))));
         IDF_Table(i,j) = (yT(i)/Alpha_Rainfall(j)) + u_Rainfall(j);
    end
end 

%% Plotting Original IDF Table

% Plot non-smoothed IDF Curves

% IDF_Plot(Duration,IDF_Table); % Plot Initial Data withouth smoothing them

% Smooth Original IDF Curves

[Duration5,IDF5] = smoothIDF(Duration,IDF_Table(1,:)); 
[~,IDF10]        = smoothIDF(Duration,IDF_Table(2,:)); 
[~,IDF20]        = smoothIDF(Duration,IDF_Table(3,:)); 
[~,IDF30]        = smoothIDF(Duration,IDF_Table(4,:));
[~,IDF40]        = smoothIDF(Duration,IDF_Table(5,:));
[~,IDF50]        = smoothIDF(Duration,IDF_Table(6,:));
[~,IDF60]        = smoothIDF(Duration,IDF_Table(7,:));
[~,IDF75]        = smoothIDF(Duration,IDF_Table(8,:));
[~,IDF100]       = smoothIDF(Duration,IDF_Table(9,:));

IDF_Table_Smoothed  = [IDF5;IDF10;IDF20;IDF30;IDF40;IDF50;IDF60;IDF75;IDF100];
Duration_Smoothed   = Duration5;

%% Creating IDF Numerical Model Bernard (1932)

% Linearization of Variables

Log_IDF         = log10(IDF_Table_Smoothed);
Log_IDF         = reshape(Log_IDF,(length(IDF_Table_Smoothed(1,:))*length(IDF_Table_Smoothed(:,1))),1);

Log_Duration    = log10(Duration_Smoothed*60); % Converting from hours to minutes
Log_Duration    = [Log_Duration;Log_Duration;Log_Duration;Log_Duration;Log_Duration;Log_Duration;Log_Duration;Log_Duration;Log_Duration];
Log_Duration    = reshape(Log_Duration,(length(Log_Duration(1,:))*length(Log_Duration(:,1))),1);

Log_T           = log10(Return_Period);  
Log_T           = repmat(Log_T,length(IDF_Table_Smoothed(1,:)));
Log_T           = Log_T(:,1);

Predictors      = [Log_T Log_Duration];
Regression      = LinearModel.fit(Predictors,Log_IDF);

a0              = 10^(double(Regression.Coefficients(1,1)));
a1              = double(Regression.Coefficients(2,1));
a2              = -1*(double(Regression.Coefficients(3,1)));

k              = num2str(a0);
a              = num2str(a1);
b              = num2str(a2);

% Defining Final Bernard Model (1932)

syms I D T;              % Define symbolic variables
Model = (k*T^a/D^b);     % Create symbolic equation
fprintf('The Bernard Model is I = %s = (mm/min)\n',char(Model));

%% Validate Model

nn           = length(Log_IDF(:,1));
IDF_Real     = 10.^(Log_IDF(:,1));
T            = 10.^(Log_T);
D            = 10.^(Log_Duration);
I            = zeros(nn,1);

for i = 1:nn;
            I(i) = (a0*T(i).^a1/D(i).^a2);      % Running Model
end 

Validation      = LinearModel.fit(I,IDF_Real);  % Validate Model

MSE             = Validation.MSE;               % Fitting Measures
MSE             = num2str(MSE);

RMSE            = Validation.RMSE;
RMSE            = num2str(RMSE);

R2              = Validation.Rsquared;
R2              = R2.Ordinary;
R2              = num2str(R2);

%% Final Plots (from smoothed IDF)

% IDF_Plot(Duration_Smoothed,IDF_Table_Smoothed); % Plot Smoothed IDF Curves
IDF_Plot2(Duration_Smoothed,IDF_Table_Smoothed);% Plot Smoothed IDF Curves 
                                                % showing breaking points 
                                                % [1,2,6,12 and 24 hours], 
                                                % Model and Fitting Measures
                                                % are also included
xlim=get(gca,'XLim');
ylim=get(gca,'YLim');

text(0.8*xlim(1)+0.68*xlim(2),0.3*ylim(1)+0.56*ylim(2),...
['\fontname{times}{I} = (',k,'*{T}^',a,')/({D}^',b,')'],'FontSize',11,...
'EdgeColor',[0 0 0]);

text(0.8*xlim(1)+0.68*xlim(2),0.3*ylim(1)+0.51*ylim(2),...
['\fontname{times}{R}^2 = ',R2,],'FontSize',11,...
'EdgeColor',[0 0 0]);

text(0.8*xlim(1)+0.68*xlim(2),0.3*ylim(1)+0.46*ylim(2),...
['\fontname{times}{MSE} = ',MSE,],'FontSize',11,...
'EdgeColor',[0 0 0]);

text(0.8*xlim(1)+0.68*xlim(2),0.3*ylim(1)+0.41*ylim(2),...
['\fontname{times}{RMSE} = ',RMSE,],'FontSize',11,...
'EdgeColor',[0 0 0]);

%% Clear Unused Variables

clear IDF5 IDF10 IDF20 IDF30 IDF40 IDF50 IDF60 IDF75 IDF100 Duration5
clear D I Model Regression T Log_Duration Log_IDF Log_T Predictors
clear a0 a1 a2 i j m n t Duration_Smoothed IDF_Table_Smoothed IDF_Real
clear MSE RMSE R2 Validation nn xlim ylim











